@@ -1,5 +1,5 @@
 ############################################################################
-# Copyright (C) 2016  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2016, 2017  Internet Systems Consortium, Inc. ("ISC")
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -68,7 +68,7 @@ class PolicyLex:
         return t
 
     def t_ALGNAME(self, t):
-        r'(?i)\b(RSAMD5|DH|DSA|NSEC3DSA|ECC|RSASHA1|NSEC3RSASHA1|RSASHA256|RSASHA512|ECCGOST|ECDSAP256SHA256|ECDSAP384SHA384)\b'
+        r'(?i)\b(RSAMD5|DH|DSA|NSEC3DSA|ECC|RSASHA1|NSEC3RSASHA1|RSASHA256|RSASHA512|ECCGOST|ECDSAP256SHA256|ECDSAP384SHA384|ED25519|ED448)\b'
         t.value = t.value.upper()
         return t
 
@@ -131,14 +131,16 @@ class Policy:
     directory = None
     valid_key_sz_per_algo = {'DSA': [512, 1024],
                              'NSEC3DSA': [512, 1024],
-                             'RSAMD5': [512, 4096],
-                             'RSASHA1': [512, 4096],
+                             'RSAMD5': [1024, 4096],
+                             'RSASHA1': [1024, 4096],
                              'NSEC3RSASHA1': [512, 4096],
-                             'RSASHA256': [512, 4096],
-                             'RSASHA512': [512, 4096],
+                             'RSASHA256': [1024, 4096],
+                             'RSASHA512': [1024, 4096],
                              'ECCGOST': None,
                              'ECDSAP256SHA256': None,
-                             'ECDSAP384SHA384': None}
+                             'ECDSAP384SHA384': None,
+                             'ED25519': None,
+                             'ED448': None}
 
     def __init__(self, name=None, algorithm=None, parent=None):
         self.name = name
@@ -275,7 +277,9 @@ class Policy:
 
             if self.algorithm in ['ECCGOST', \
                                   'ECDSAP256SHA256', \
-                                  'ECDSAP384SHA384']:
+                                  'ECDSAP384SHA384', \
+                                  'ED25519', \
+                                  'ED448']:
                 self.ksk_keysize = None
                 self.zsk_keysize = None
 
@@ -378,6 +382,18 @@ class dnssec_policy:
         self.alg_policy['ECDSAP384SHA384'].ksk_keysize = None;
         self.alg_policy['ECDSAP384SHA384'].zsk_keysize = None;
 
+        self.alg_policy['ED25519'] = copy(p)
+        self.alg_policy['ED25519'].algorithm = "ED25519"
+        self.alg_policy['ED25519'].name = "ED25519"
+        self.alg_policy['ED25519'].ksk_keysize = None;
+        self.alg_policy['ED25519'].zsk_keysize = None;
+
+        self.alg_policy['ED448'] = copy(p)
+        self.alg_policy['ED448'].algorithm = "ED448"
+        self.alg_policy['ED448'].name = "ED448"
+        self.alg_policy['ED448'].ksk_keysize = None;
+        self.alg_policy['ED448'].zsk_keysize = None;
+
         if filename:
             self.load(filename)
 
@@ -522,6 +538,16 @@ class dnssec_policy:
         p[0] = p[1]
         pass
 
+    def p_domain(self, p):
+        '''domain : STR
+                  | QSTRING
+                  | KEYTYPE
+                  | DATESUFFIX'''
+        p[0] = p[1].strip()
+        if not re.match(r'^[\w.-][\w.-]*$', p[0]):
+            raise PolicyException('invalid domain')
+        pass
+
     def p_new_policy(self, p):
         "new_policy :"
         self.current = Policy()
@@ -534,7 +560,7 @@ class dnssec_policy:
         pass
 
     def p_zone_policy(self, p):
-        "zone_policy : ZONE name new_policy policy_option_group SEMI"
+        "zone_policy : ZONE domain new_policy policy_option_group SEMI"
         self.current.name = p[2].rstrip('.')
         self.current.is_zone = True
         self.zone_policy[p[2].rstrip('.').lower()] = self.current
